@@ -6,20 +6,19 @@ use warnings;
 
 BEGIN {
 	$Type::Coercion::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Coercion::VERSION   = '0.022';
+	$Type::Coercion::VERSION   = '0.044';
 }
 
 use Eval::TypeTiny qw<>;
 use Scalar::Util qw< blessed >;
 use Types::TypeTiny qw<>;
 
-sub _croak ($;@) { require Type::Exception; goto \&Type::Exception::croak }
+sub _croak ($;@) { require Error::TypeTiny; goto \&Error::TypeTiny::croak }
 
 use overload
 	q("")      => sub { caller =~ m{^(Moo::HandleMoose|Sub::Quote)} ? overload::StrVal($_[0]) : $_[0]->display_name },
 	q(bool)    => sub { 1 },
 	q(&{})     => "_overload_coderef",
-	q(+)       => sub { __PACKAGE__->add(@_) },
 	fallback   => 1,
 ;
 
@@ -57,14 +56,15 @@ sub new
 	
 	my $self = bless \%params, $class;
 	$self->add_type_coercions(@$C) if @$C;
+	$self->_preserve_type_constraint;
 	Scalar::Util::weaken($self->{type_constraint}); # break ref cycle
 	$self->{frozen} = $F if $F;
 	
 	unless ($self->is_anon)
 	{
 		# First try a fast ASCII-only expression, but fall back to Unicode
-		$self->name =~ /^[A-Z][A-Za-z0-9_]+$/sm
-			or eval q( use 5.008; $self->name =~ /^\p{Lu}[\p{L}0-9_]+$/sm )
+		$self->name =~ /^_{0,2}[A-Z][A-Za-z0-9_]+$/sm
+			or eval q( use 5.008; $self->name =~ /^_{0,2}\p{Lu}[\p{L}0-9_]+$/sm )
 			or _croak '"%s" is not a valid coercion name', $self->name;
 	}
 	
@@ -74,19 +74,37 @@ sub new
 sub name                   { $_[0]{name} }
 sub display_name           { $_[0]{display_name}      ||= $_[0]->_build_display_name }
 sub library                { $_[0]{library} }
-sub type_constraint        { $_[0]{type_constraint} }
+sub type_constraint        { $_[0]{type_constraint}   ||= $_[0]->_maybe_restore_type_constraint }
 sub type_coercion_map      { $_[0]{type_coercion_map} ||= [] }
 sub moose_coercion         { $_[0]{moose_coercion}    ||= $_[0]->_build_moose_coercion }
 sub compiled_coercion      { $_[0]{compiled_coercion} ||= $_[0]->_build_compiled_coercion }
 sub frozen                 { $_[0]{frozen}            ||= 0 }
 sub coercion_generator     { $_[0]{coercion_generator} }
 sub parameters             { $_[0]{parameters} }
+sub parameterized_from     { $_[0]{parameterized_from} }
 
 sub has_library            { exists $_[0]{library} }
-sub has_type_constraint    { defined $_[0]{type_constraint} } # sic
+sub has_type_constraint    { defined $_[0]->type_constraint } # sic
 sub has_coercion_generator { exists $_[0]{coercion_generator} }
 sub has_parameters         { exists $_[0]{parameters} }
 
+sub _preserve_type_constraint
+{
+	my $self = shift;
+	$self->{_compiled_type_constraint_check} = $self->{type_constraint}->compiled_check
+		if $self->{type_constraint};
+}
+
+sub _maybe_restore_type_constraint
+{
+	my $self = shift;
+	if ( my $check = $self->{_compiled_type_constraint_check} )
+	{
+		return Type::Tiny->new(constraint => $check);
+	}
+	return;
+}
+
 sub add
 {
 	my $class = shift;
@@ -97,9 +115,9 @@ sub add
 	
 	_croak "Attempt to add $class to something that is not a $class"
 		unless blessed($x) && blessed($y) && $x->isa($class) && $y->isa($class);
-
+	
 	($y, $x) = ($x, $y) if $swap;
-
+	
 	my %opts;
 	if ($x->has_type_constraint and $y->has_type_constraint and $x->type_constraint == $y->type_constraint)
 	{
@@ -386,7 +404,7 @@ sub _codelike_type_coercion_map
 		}
 		else
 		{
-			Eval::TypeTiny::eval_closure(
+			push @new, Eval::TypeTiny::eval_closure(
 				source      => sprintf('sub { local $_ = $_[0]; %s }', $converter),
 				description => sprintf("temporary compiled converter from '%s'", $type),
 			);
@@ -420,6 +438,24 @@ sub parameterize
 		type_coercion_map  => [ $self->coercion_generator->($self, $self->type_constraint, @_) ],
 		parameters         => \@_,
 		frozen             => 1,
+		parameterized_from => $self,
+	);
+}
+
+sub _reparameterize
+{
+	my $self = shift;
+	my ($target_type) = @_;
+	
+	$self->is_parameterized or return $self;
+	my $parent = $self->parameterized_from;
+	
+	return ref($self)->new(
+		type_constraint    => $target_type,
+		type_coercion_map  => [ $parent->coercion_generator->($parent, $target_type, @{$self->parameters}) ],
+		parameters         => \@_,
+		frozen             => 1,
+		parameterized_from => $parent,
 	);
 }
 
@@ -504,6 +540,11 @@ __END__
 
 Type::Coercion - a set of coercions to a particular target type constraint
 
+=head1 STATUS
+
+This module is covered by the
+L<Type-Tiny stability policy|Type::Tiny::Manual::Policies/"STABILITY">.
+
 =head1 DESCRIPTION
 
 =head2 Constructors
@@ -522,8 +563,35 @@ Create a Type::Coercion from two existing Type::Coercion objects.
 
 =head2 Attributes
 
+Attributes are named values that may be passed to the constructor. For
+each attribute, there is a corresponding reader method. For example:
+
+   my $c = Type::Coercion->new( type_constraint => Int );
+   my $t = $c->type_constraint;  # Int
+
+=head3 Important attributes
+
+These are the attributes you are likely to be most interested in
+providing when creating your own type coercions, and most interested
+in reading when dealing with coercion objects.
+
 =over
 
+=item C<type_constraint>
+
+Weak reference to the target type constraint (i.e. the type constraint which
+the output of coercion coderefs is expected to conform to).
+
+=item C<type_coercion_map>
+
+Arrayref of source-type/code pairs. Don't set this in the constructor; use
+the C<add_type_coercions> method instead.
+
+=item C<frozen>
+
+Boolean; default false. A frozen coercion cannot have C<add_type_coercions>
+called upon it.
+
 =item C<name>
 
 A name for the coercion. These need to conform to certain naming
@@ -544,15 +612,30 @@ The package name of the type library this coercion is associated with.
 Optional. Informational only: setting this attribute does not install
 the coercion into the package.
 
-=item C<type_constraint>
+=back
 
-Weak reference to the target type constraint (i.e. the type constraint which
-the output of coercion coderefs is expected to conform to).
+=head3 Attributes related to parameterizable and parameterized coercions
 
-=item C<type_coercion_map>
+The following attributes are used for parameterized coercions, but are not
+fully documented because they may change in the near future:
 
-Arrayref of source-type/code pairs. Don't set this in the constructor; use
-the C<add_type_coercions> method instead.
+=over
+
+=item C<< coercion_generator >>
+
+=item C<< parameters >>
+
+=item C<< parameterized_from >>
+
+=back
+
+=head3 Lazy generated attributes
+
+The following attributes should not be usually passed to the constructor;
+unless you're doing something especially unusual, you should rely on the
+default lazily-built return values.
+
+=over
 
 =item C<< compiled_coercion >>
 
@@ -567,39 +650,49 @@ pretty fast coderef, inlining all type constraint checks, etc.
 A L<Moose::Meta::TypeCoercion> object equivalent to this one. Don't set this
 manually; rely on the default built one.
 
-=item C<frozen>
-
-Boolean; default false. A frozen coercion cannot have C<add_type_coercions>
-called upon it.
-
 =back
 
 =head2 Methods
 
+=head3 Predicate methods
+
+These methods return booleans indicating information about the coercion.
+They are each tightly associated with a particular attribute.
+(See L</"Attributes">.)
+
 =over
 
 =item C<has_type_constraint>, C<has_library>
 
-Predicate methods.
+Simple Moose-style predicate methods indicating the presence or
+absence of an attribute.
 
 =item C<is_anon>
 
 Returns true iff the coercion does not have a C<name>.
 
-=item C<< qualified_name >>
+=back
 
-For non-anonymous coercions that have a library, returns a qualified
-C<< "Library::Type" >> sort of name. Otherwise, returns the same as C<name>.
+The following predicates are used for parameterized coercions, but are not
+fully documented because they may change in the near future:
 
-=item C<< add_type_coercions($type1, $code1, ...) >>
+=over
 
-Takes one or more pairs of L<Type::Tiny> constraints and coercion code,
-creating an ordered list of source types and coercion codes.
+=item C<< has_coercion_generator >>
 
-Coercion codes can be expressed as either a string of Perl code (this
-includes objects which overload stringification), or a coderef (or object
-that overloads coderefification). In either case, the value to be coerced
-is C<< $_ >>.
+=item C<< has_parameters >>
+
+=item C<< is_parameterizable >>
+
+=item C<< is_parameterized >>
+
+=back
+
+=head3 Coercion
+
+The following methods are used for coercing values to a type constraint:
+
+=over
 
 =item C<< coerce($value) >>
 
@@ -615,6 +708,47 @@ does not validate against the target type constraint.
 
 Returns the coerced value.
 
+=back
+
+=head3 Coercion code definition methods
+
+=over
+
+=item C<< add_type_coercions($type1, $code1, ...) >>
+
+Takes one or more pairs of L<Type::Tiny> constraints and coercion code,
+creating an ordered list of source types and coercion codes.
+
+Coercion codes can be expressed as either a string of Perl code (this
+includes objects which overload stringification), or a coderef (or object
+that overloads coderefification). In either case, the value to be coerced
+is C<< $_ >>.
+
+=item C<< freeze >>
+
+Sets the C<frozen> attribute to true. There is no C<unfreeze>. Called
+automatically by L<Type::Tiny> sometimes.
+
+=back
+
+=head3 Parameterization
+
+The following method is used for parameterized coercions, but is not
+fully documented because it may change in the near future:
+
+=over
+
+=item C<< parameterize(@params) >>
+
+=back
+
+=head3 Type coercion introspection methods
+
+These methods allow you to determine a coercion's relationship to type
+constraints:
+
+=over
+
 =item C<< has_coercion_for_type($source_type) >>
 
 Returns true iff this coercion has a coercion from the source type.
@@ -632,6 +766,20 @@ Returns the special string C<< "0 but true" >> if no coercion would be
 actually be necessary for this value (due to it already meeting the target
 type constraint).
 
+=back
+
+The C<type_constraint> attribute provides a type constraint object for the
+target type constraint of the coercion. See L</"Attributes">.
+
+=head3 Inlining methods
+
+=for stopwords uated
+
+The following methods are used to generate strings of Perl code which
+may be pasted into stringy C<eval>uated subs to perform type coercions:
+
+=over
+
 =item C<< can_be_inlined >>
 
 Returns true iff the coercion can be inlined.
@@ -640,10 +788,17 @@ Returns true iff the coercion can be inlined.
 
 Much like C<inline_coerce> from L<Type::Tiny>.
 
-=item C<< freeze >>
+=back
 
-Set C<frozen> to true. There is no C<unfreeze>. Called automatically by
-L<Type::Tiny> sometimes.
+=head3 Other methods
+
+=over
+
+=item C<< qualified_name >>
+
+For non-anonymous coercions that have a library, returns a qualified
+C<< "MyLib::MyCoercion" >> sort of name. Otherwise, returns the same
+as C<name>.
 
 =item C<< isa($class) >>, C<< can($method) >>, C<< AUTOLOAD(@args) >>
 
@@ -652,27 +807,6 @@ a Moose::Meta::TypeCoercion.
 
 =back
 
-The following methods are used for parameterized coercions, but are not
-fully documented because they may change in the near future:
-
-=over
-
-=item C<< coercion_generator >>
-
-=item C<< has_coercion_generator >>
-
-=item C<< has_parameters >>
-
-=item C<< is_parameterizable >>
-
-=item C<< is_parameterized >>
-
-=item C<< parameterize(@params) >>
-
-=item C<< parameters >>
-
-=back
-
 The following methods exist for Moose/Mouse compatibility, but do not do
 anything useful.
 
@@ -700,12 +834,11 @@ Coderefification is overloaded to call C<coerce>.
 
 On Perl 5.10.1 and above, smart match is overloaded to call C<has_coercion_for_value>.
 
-=item *
-
-Addition is overloaded to call C<add>.
-
 =back
 
+Previous versions of Type::Coercion would overload the C<< + >> operator
+to call C<add>. Support for this was dropped after 0.040.
+
 =head1 DIAGNOSTICS
 
 =over
@@ -765,7 +898,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013 by Toby Inkster.
+This software is copyright (c) 2013-2014 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
