@@ -6,19 +6,20 @@ use warnings;
 
 BEGIN {
 	$Type::Tiny::Duck::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Tiny::Duck::VERSION   = '0.022';
+	$Type::Tiny::Duck::VERSION   = '0.044';
 }
 
 use Scalar::Util qw< blessed >;
 
-sub _croak ($;@) { require Type::Exception; goto \&Type::Exception::croak }
+sub _croak ($;@) { require Error::TypeTiny; goto \&Error::TypeTiny::croak }
 
-use base "Type::Tiny";
+require Type::Tiny;
+our @ISA = 'Type::Tiny';
 
 sub new {
 	my $proto = shift;
 	
-	my %opts = @_;
+	my %opts = (@_==1) ? %{$_[0]} : @_;
 	_croak "Duck type constraints cannot have a parent constraint passed to the constructor" if exists $opts{parent};
 	_croak "Duck type constraints cannot have a constraint coderef passed to the constructor" if exists $opts{constraint};
 	_croak "Duck type constraints cannot have a inlining coderef passed to the constructor" if exists $opts{inlined};
@@ -48,18 +49,15 @@ sub _build_inlined
 	sub {
 		my $var = $_[1];
 		local $" = q{ };
-		qq{ Scalar::Util::blessed($var) and not grep(!$var->can(\$_), qw/@methods/) };
+		# If $var is $_ or $_->{foo} or $foo{$_} or somesuch, then we
+		# can't use it within the grep expression, so we need to save
+		# it into a temporary variable ($tmp).
+		($var =~ /\$_/)
+			? qq{ Scalar::Util::blessed($var) and not do { my \$tmp = $var; grep(!\$tmp->can(\$_), qw/@methods/) } }
+			: qq{ Scalar::Util::blessed($var) and not grep(!$var->can(\$_), qw/@methods/) };
 	};
 }
 
-sub _build_default_message
-{
-	my $self = shift;
-	return sub { sprintf 'value "%s" did not pass type constraint', $_[0] } if $self->is_anon;
-	my $name = "$self";
-	return sub { sprintf 'value "%s" did not pass type constraint "%s"', $_[0], $name };
-}
-
 sub _instantiate_moose_type
 {
 	my $self = shift;
@@ -83,6 +81,28 @@ sub parent
 	Types::Standard::Object();
 }
 
+sub validate_explain
+{
+	my $self = shift;
+	my ($value, $varname) = @_;
+	$varname = '$_' unless defined $varname;
+	
+	return undef if $self->check($value);
+	return ["Not a blessed reference"] unless blessed($value);
+	
+	require Type::Utils;
+	return [
+		sprintf(
+			'"%s" requires that the reference can %s',
+			$self,
+			Type::Utils::english_list(map qq["$_"], @{$self->methods}),
+		),
+		map  sprintf('The reference cannot "%s"', $_),
+		grep !$value->can($_),
+		@{$self->methods}
+	];
+}
+
 1;
 
 __END__
@@ -95,6 +115,11 @@ __END__
 
 Type::Tiny::Duck - type constraints based on the "can" method
 
+=head1 STATUS
+
+This module is covered by the
+L<Type-Tiny stability policy|Type::Tiny::Manual::Policies/"STABILITY">.
+
 =head1 DESCRIPTION
 
 Type constraints of the general form C<< { $_->can("method") } >>.
@@ -112,13 +137,13 @@ An arrayref of method names.
 
 =item C<constraint>
 
-Unlike Type::Tiny, you should generally I<not> pass a constraint to the
-constructor. Instead rely on the default.
+Unlike Type::Tiny, you I<cannot> pass a constraint coderef to the constructor.
+Instead rely on the default.
 
 =item C<inlined>
 
-Unlike Type::Tiny, you should generally I<not> pass an inlining coderef to
-the constructor. Instead rely on the default.
+Unlike Type::Tiny, you I<cannot> pass an inlining coderef to the constructor.
+Instead rely on the default.
 
 =item C<parent>
 
@@ -146,7 +171,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013 by Toby Inkster.
+This software is copyright (c) 2013-2014 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
