@@ -6,20 +6,21 @@ use warnings;
 
 BEGIN {
 	$Type::Tiny::Enum::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Tiny::Enum::VERSION   = '0.022';
+	$Type::Tiny::Enum::VERSION   = '0.044';
 }
 
-sub _croak ($;@) { require Type::Exception; goto \&Type::Exception::croak }
+sub _croak ($;@) { require Error::TypeTiny; goto \&Error::TypeTiny::croak }
 
 use overload q[@{}] => 'values';
 
-use base "Type::Tiny";
+require Type::Tiny;
+our @ISA = 'Type::Tiny';
 
 sub new
 {
 	my $proto = shift;
 	
-	my %opts = @_;
+	my %opts = (@_==1) ? %{$_[0]} : @_;
 	_croak "Enum type constraints cannot have a parent constraint passed to the constructor" if exists $opts{parent};
 	_croak "Enum type constraints cannot have a constraint coderef passed to the constructor" if exists $opts{constraint};
 	_croak "Enum type constraints cannot have a inlining coderef passed to the constructor" if exists $opts{inlined};
@@ -59,8 +60,8 @@ sub inline_check
 	my $self = shift;
 	my $regexp = join "|", map quotemeta, @$self;
 	$_[0] eq '$_'
-		? "(defined and m{\\A(?:$regexp)\\z})"
-		: "(defined($_[0]) and $_[0] =~ m{\\A(?:$regexp)\\z})";
+		? "(defined and !ref and m{\\A(?:$regexp)\\z})"
+		: "(defined($_[0]) and !ref($_[0]) and $_[0] =~ m{\\A(?:$regexp)\\z})";
 }
 
 sub _instantiate_moose_type
@@ -85,6 +86,37 @@ sub parent
 	Types::Standard::Str();
 }
 
+sub validate_explain
+{
+	my $self = shift;
+	my ($value, $varname) = @_;
+	$varname = '$_' unless defined $varname;
+	
+	return undef if $self->check($value);
+	
+	require Type::Utils;
+	!defined($value) ? [
+		sprintf(
+			'"%s" requires that the value is defined',
+			$self,
+		),
+	] :
+	@$self < 13 ? [
+		sprintf(
+			'"%s" requires that the value is equal to %s',
+			$self,
+			Type::Utils::english_list(\"or", map B::perlstring($_), @$self),
+		),
+	] :
+	[
+		sprintf(
+			'"%s" requires that the value is one of an enumerated list of strings',
+			$self,
+		),
+	];
+}
+
+
 1;
 
 __END__
@@ -97,6 +129,11 @@ __END__
 
 Type::Tiny::Enum - string enum type constraints
 
+=head1 STATUS
+
+This module is covered by the
+L<Type-Tiny stability policy|Type::Tiny::Manual::Policies/"STABILITY">.
+
 =head1 DESCRIPTION
 
 Enum type constraints.
@@ -115,13 +152,13 @@ overloading) will be stringified in the constructor.
 
 =item C<constraint>
 
-Unlike Type::Tiny, you should generally I<not> pass a constraint to the
-constructor. Instead rely on the default.
+Unlike Type::Tiny, you I<cannot> pass a constraint coderef to the constructor.
+Instead rely on the default.
 
 =item C<inlined>
 
-Unlike Type::Tiny, you should generally I<not> pass an inlining coderef to
-the constructor. Instead rely on the default.
+Unlike Type::Tiny, you I<cannot> pass an inlining coderef to the constructor.
+Instead rely on the default.
 
 =item C<parent>
 
@@ -159,7 +196,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013 by Toby Inkster.
+This software is copyright (c) 2013-2014 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
